@@ -38,7 +38,7 @@ import (
 	"github.com/prometheus/prometheus/pkg/timestamp"
 	"github.com/prometheus/prometheus/prompb"
 	"github.com/prometheus/prometheus/promql"
-	"github.com/prometheus/prometheus/retrieval"
+	"github.com/prometheus/prometheus/scrape"
 	"github.com/prometheus/prometheus/storage"
 	"github.com/prometheus/prometheus/storage/remote"
 	"github.com/prometheus/prometheus/util/httputil"
@@ -82,11 +82,13 @@ func (e *apiError) Error() string {
 }
 
 type targetRetriever interface {
-	Targets() []*retrieval.Target
+	Targets() []*scrape.Target
+	DroppedTargets() []*scrape.Target
 }
 
 type alertmanagerRetriever interface {
 	Alertmanagers() []*url.URL
+	DroppedAlertmanagers() []*url.URL
 }
 
 type response struct {
@@ -108,15 +110,16 @@ type apiFunc func(r *http.Request) (interface{}, *apiError)
 // API can register a set of endpoints in a router and handle
 // them using the provided storage and query engine.
 type API struct {
-	Queryable   promql.Queryable
+	Queryable   storage.Queryable
 	QueryEngine *promql.Engine
 
 	targetRetriever       targetRetriever
 	alertmanagerRetriever alertmanagerRetriever
 
-	now    func() time.Time
-	config func() config.Config
-	ready  func(http.HandlerFunc) http.HandlerFunc
+	now      func() time.Time
+	config   func() config.Config
+	flagsMap map[string]string
+	ready    func(http.HandlerFunc) http.HandlerFunc
 
 	db          func() *tsdb.DB
 	enableAdmin bool
@@ -125,10 +128,11 @@ type API struct {
 // NewAPI returns an initialized API type.
 func NewAPI(
 	qe *promql.Engine,
-	q promql.Queryable,
+	q storage.Queryable,
 	tr targetRetriever,
 	ar alertmanagerRetriever,
 	configFunc func() config.Config,
+	flagsMap map[string]string,
 	readyFunc func(http.HandlerFunc) http.HandlerFunc,
 	db func() *tsdb.DB,
 	enableAdmin bool,
@@ -140,6 +144,7 @@ func NewAPI(
 		alertmanagerRetriever: ar,
 		now:         time.Now,
 		config:      configFunc,
+		flagsMap:    flagsMap,
 		ready:       readyFunc,
 		db:          db,
 		enableAdmin: enableAdmin,
@@ -180,6 +185,7 @@ func (api *API) Register(r *route.Router) {
 	r.Get("/alertmanagers", instr("alertmanagers", api.alertmanagers))
 
 	r.Get("/status/config", instr("config", api.serveConfig))
+	r.Get("/status/flags", instr("flags", api.serveFlags))
 	r.Post("/read", api.ready(prometheus.InstrumentHandler("read", http.HandlerFunc(api.remoteRead))))
 
 	// Admin APIs
@@ -222,7 +228,7 @@ func (api *API) query(r *http.Request) (interface{}, *apiError) {
 		defer cancel()
 	}
 
-	qry, err := api.QueryEngine.NewInstantQuery(r.FormValue("query"), ts)
+	qry, err := api.QueryEngine.NewInstantQuery(api.Queryable, r.FormValue("query"), ts)
 	if err != nil {
 		return nil, &apiError{errorBadData, err}
 	}
@@ -296,7 +302,7 @@ func (api *API) queryRange(r *http.Request) (interface{}, *apiError) {
 		defer cancel()
 	}
 
-	qry, err := api.QueryEngine.NewRangeQuery(r.FormValue("query"), start, end, step)
+	qry, err := api.QueryEngine.NewRangeQuery(api.Queryable, r.FormValue("query"), start, end, step)
 	if err != nil {
 		return nil, &apiError{errorBadData, err}
 	}
@@ -396,7 +402,7 @@ func (api *API) series(r *http.Request) (interface{}, *apiError) {
 
 	var sets []storage.SeriesSet
 	for _, mset := range matcherSets {
-		s, err := q.Select(mset...)
+		s, err := q.Select(nil, mset...)
 		if err != nil {
 			return nil, &apiError{errorExec, err}
 		}
@@ -428,19 +434,27 @@ type Target struct {
 
 	ScrapeURL string `json:"scrapeUrl"`
 
-	LastError  string                 `json:"lastError"`
-	LastScrape time.Time              `json:"lastScrape"`
-	Health     retrieval.TargetHealth `json:"health"`
+	LastError  string              `json:"lastError"`
+	LastScrape time.Time           `json:"lastScrape"`
+	Health     scrape.TargetHealth `json:"health"`
+}
+
+// DroppedTarget has the information for one target that was dropped during relabelling.
+type DroppedTarget struct {
+	// Labels before any processing.
+	DiscoveredLabels map[string]string `json:"discoveredLabels"`
 }
 
 // TargetDiscovery has all the active targets.
 type TargetDiscovery struct {
-	ActiveTargets []*Target `json:"activeTargets"`
+	ActiveTargets  []*Target        `json:"activeTargets"`
+	DroppedTargets []*DroppedTarget `json:"droppedTargets"`
 }
 
 func (api *API) targets(r *http.Request) (interface{}, *apiError) {
 	targets := api.targetRetriever.Targets()
-	res := &TargetDiscovery{ActiveTargets: make([]*Target, len(targets))}
+	droppedTargets := api.targetRetriever.DroppedTargets()
+	res := &TargetDiscovery{ActiveTargets: make([]*Target, len(targets)), DroppedTargets: make([]*DroppedTarget, len(droppedTargets))}
 
 	for i, t := range targets {
 		lastErrStr := ""
@@ -459,12 +473,19 @@ func (api *API) targets(r *http.Request) (interface{}, *apiError) {
 		}
 	}
 
+	for i, t := range droppedTargets {
+		res.DroppedTargets[i] = &DroppedTarget{
+			DiscoveredLabels: t.DiscoveredLabels().Map(),
+		}
+	}
+
 	return res, nil
 }
 
 // AlertmanagerDiscovery has all the active Alertmanagers.
 type AlertmanagerDiscovery struct {
-	ActiveAlertmanagers []*AlertmanagerTarget `json:"activeAlertmanagers"`
+	ActiveAlertmanagers  []*AlertmanagerTarget `json:"activeAlertmanagers"`
+	DroppedAlertmanagers []*AlertmanagerTarget `json:"droppedAlertmanagers"`
 }
 
 // AlertmanagerTarget has info on one AM.
@@ -474,12 +495,14 @@ type AlertmanagerTarget struct {
 
 func (api *API) alertmanagers(r *http.Request) (interface{}, *apiError) {
 	urls := api.alertmanagerRetriever.Alertmanagers()
-	ams := &AlertmanagerDiscovery{ActiveAlertmanagers: make([]*AlertmanagerTarget, len(urls))}
-
+	droppedURLS := api.alertmanagerRetriever.DroppedAlertmanagers()
+	ams := &AlertmanagerDiscovery{ActiveAlertmanagers: make([]*AlertmanagerTarget, len(urls)), DroppedAlertmanagers: make([]*AlertmanagerTarget, len(droppedURLS))}
 	for i, url := range urls {
 		ams.ActiveAlertmanagers[i] = &AlertmanagerTarget{URL: url.String()}
 	}
-
+	for i, url := range droppedURLS {
+		ams.DroppedAlertmanagers[i] = &AlertmanagerTarget{URL: url.String()}
+	}
 	return ams, nil
 }
 
@@ -494,6 +517,10 @@ func (api *API) serveConfig(r *http.Request) (interface{}, *apiError) {
 	return cfg, nil
 }
 
+func (api *API) serveFlags(r *http.Request) (interface{}, *apiError) {
+	return api.flagsMap, nil
+}
+
 func (api *API) remoteRead(w http.ResponseWriter, r *http.Request) {
 	req, err := remote.DecodeReadRequest(r)
 	if err != nil {
@@ -537,7 +564,7 @@ func (api *API) remoteRead(w http.ResponseWriter, r *http.Request) {
 			}
 		}
 
-		set, err := querier.Select(filteredMatchers...)
+		set, err := querier.Select(nil, filteredMatchers...)
 		if err != nil {
 			http.Error(w, err.Error(), http.StatusInternalServerError)
 			return
@@ -630,6 +657,8 @@ func (api *API) snapshot(r *http.Request) (interface{}, *apiError) {
 	if !api.enableAdmin {
 		return nil, &apiError{errorUnavailable, errors.New("Admin APIs disabled")}
 	}
+	skipHead, _ := strconv.ParseBool(r.FormValue("skip_head"))
+
 	db := api.db()
 	if db == nil {
 		return nil, &apiError{errorUnavailable, errors.New("TSDB not ready")}
@@ -645,7 +674,7 @@ func (api *API) snapshot(r *http.Request) (interface{}, *apiError) {
 	if err := os.MkdirAll(dir, 0777); err != nil {
 		return nil, &apiError{errorInternal, fmt.Errorf("create snapshot directory: %s", err)}
 	}
-	if err := db.Snapshot(dir); err != nil {
+	if err := db.Snapshot(dir, !skipHead); err != nil {
 		return nil, &apiError{errorInternal, fmt.Errorf("create snapshot: %s", err)}
 	}
 
