@@ -163,6 +163,7 @@ Prometheus.Graph.prototype.initialize = function() {
   self.queryForm.submit(function() {
     self.consoleTab.addClass("reload");
     self.graphTab.addClass("reload");
+    self.handleChange();
     self.submitQuery();
     return false;
   });
@@ -192,8 +193,6 @@ Prometheus.Graph.prototype.initialize = function() {
   }
   self.populateInsertableMetrics();
 
-  queryHistory.bindHistoryEvents(self);
-
   if (self.expr.val()) {
     self.submitQuery();
   }
@@ -260,10 +259,9 @@ Prometheus.Graph.prototype.populateInsertableMetrics = function() {
 };
 
 Prometheus.Graph.prototype.initTypeahead = function(self) {
-  const historyIsChecked = $("div.query-history").hasClass("is-checked");
-  const source = historyIsChecked ? pageConfig.allMetrics.concat(JSON.parse(localStorage.getItem("history"))) : pageConfig.allMetrics;
-
+  const source = queryHistory.isEnabled() ? pageConfig.queryHistMetrics.concat(pageConfig.allMetrics) : pageConfig.allMetrics;
   self.expr.typeahead({
+    autoSelect: false,
     source,
     items: "all",
     matcher: function (item) {
@@ -298,6 +296,7 @@ Prometheus.Graph.prototype.initTypeahead = function(self) {
   // This needs to happen after attaching the typeahead plugin, as it
   // otherwise breaks the typeahead functionality.
   self.expr.focus();
+  queryHistory.bindHistoryEvents(self);
 }
 
 Prometheus.Graph.prototype.getOptions = function() {
@@ -446,6 +445,7 @@ Prometheus.Graph.prototype.submitQuery = function() {
           self.showError(json.error);
           return;
         }
+
         queryHistory.handleHistory(self);
         success(json.data, textStatus);
       },
@@ -829,7 +829,9 @@ Prometheus.Graph.prototype.formatKMBT = function(y) {
  * Page
 */
 const pageConfig = {
-  graphs: []
+  allMetrics: [],
+  graphs: [],
+  queryHistMetrics: JSON.parse(localStorage.getItem('history')) || [],
 };
 
 Prometheus.Page = function() {};
@@ -862,7 +864,6 @@ Prometheus.Page.prototype.addGraph = function(options) {
     this.removeGraph.bind(this)
   );
 
-  // this.graphs.push(graph);
   pageConfig.graphs.push(graph);
 
   $(window).resize(function() {
@@ -994,12 +995,16 @@ function redirectToMigratedURL() {
  * Query History helper functions
  * **/
 const queryHistory = {
+  isEnabled: function() {
+    return JSON.parse(localStorage.getItem('enable-query-history'))
+  },
+
   bindHistoryEvents: function(graph) {
     const targetEl = $('div.query-history');
     const icon = $(targetEl).children('i');
     targetEl.off('click');
 
-    if (JSON.parse(localStorage.getItem('enable-query-history'))) {
+    if (queryHistory.isEnabled()) {
       this.toggleOn(targetEl);
     }
 
@@ -1030,12 +1035,14 @@ const queryHistory = {
     parsedQueryHistory = parsedQueryHistory.slice(queryCount - 50, queryCount);
 
     localStorage.setItem('history', JSON.stringify(parsedQueryHistory));
-
-    this.updateTypeaheadMetricSet(parsedQueryHistory);
+    pageConfig.queryHistMetrics = parsedQueryHistory;
+    if (queryHistory.isEnabled()) {
+      this.updateTypeaheadMetricSet(pageConfig.queryHistMetrics.concat(pageConfig.allMetrics));
+    }
   },
 
   toggleOn: function(targetEl) {
-    this.updateTypeaheadMetricSet(JSON.parse(localStorage.getItem('history')));
+    this.updateTypeaheadMetricSet(pageConfig.queryHistMetrics.concat(pageConfig.allMetrics));
 
     $(targetEl).children('i').removeClass('glyphicon-unchecked').addClass('glyphicon-check');
     targetEl.addClass('is-checked');
@@ -1043,7 +1050,7 @@ const queryHistory = {
   },
 
   toggleOff: function(targetEl) {
-    this.updateTypeaheadMetricSet();
+    this.updateTypeaheadMetricSet(pageConfig.allMetrics);
 
     $(targetEl).children('i').removeClass('glyphicon-check').addClass('glyphicon-unchecked');
     targetEl.removeClass('is-checked');
@@ -1052,7 +1059,9 @@ const queryHistory = {
 
   updateTypeaheadMetricSet: function(metricSet) {
     pageConfig.graphs.forEach(function(graph) {
-      graph.expr.data('typeahead').source = metricSet ? pageConfig.allMetrics.concat(metricSet) : pageConfig.allMetrics;
+      if (graph.expr.data('typeahead')) {
+        graph.expr.data('typeahead').source = metricSet;
+      }
     });
   }
 };
@@ -1094,4 +1103,3 @@ function init() {
 }
 
 $(init);
-
